<?php
/**
 * 2007-2019 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    PrestaShop SA <contact@prestashop.com>
 *  @copyright 2007-2019 PrestaShop SA
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

class CaisseEnregistreuse extends Module
{
    protected $config_form = false;
    
    public function __construct()
    {
        $this->name = 'caisseEnregistreuse';
        $this->tab = 'billing_invoicing';
        $this->version = '1.0.1';
        $this->author = 'Net-assembly';
        $this->need_instance = 1;
        
        /**
         * Set $this->bootstrap to true if your module is compliant with bootstrap (PrestaShop 1.6)
         */
        $this->bootstrap = true;
        
        parent::__construct();
        
        $this->displayName = $this->l('Cash register POS');
        $this->description = $this->l('Centralize your sales in your cash register point of sale software.');//Centralisez vos ventes dans un logiciel de caisse.
        
        $this->ps_versions_compliancy = array('min' => '1.6', 'max' => _PS_VERSION_);
    }
    
    /**
     * Don't forget to create update methods if needed:
     * http://doc.prestashop.com/display/PS16/Enabling+the+Auto-Update
     */
    public function install()
    {
        //Configuration::updateValue('CAISSEENREGISTREUSE_LIVE_MODE', true);
        //Configuration::updateValue('CAISSEENREGISTREUSE_DEBUG_MODE', true);
        // actionUpdateQuantity actionCustomerAccountUpdate actionCustomerAccountAdd actionProductSave actionPaymentConfirmation
        //hookActionValidateOrder hookDisplayPayment
        $this->logData("Install");
        return parent::install() &&
        $this->registerHook('header') &&
        $this->registerHook('actionValidateOrder') /*&&
        $this->registerHook('actionCustomerAccountUpdate') &&
        $this->registerHook('actionCustomerAccountAdd') &&
        $this->registerHook('actionPaymentConfirmation') &&
        $this->registerHook('backOfficeHeader') &&
        $this->registerHook('displayPayment') &&
        $this->registerHook('displayPaymentReturn')*/;
    }
    
    public function uninstall()
    {
        //Configuration::deleteByName('CAISSEENREGISTREUSE_LIVE_MODE');
        //Configuration::deleteByName('CAISSEENREGISTREUSE_DEBUG_MODE');
        Configuration::deleteByName('CAISSEENREGISTREUSE_ACCOUNT_API');
        Configuration::deleteByName('CAISSEENREGISTREUSE_ACCOUNT_SHOPID');
        
        return parent::uninstall();
    }
    
    /**
     * Load the configuration form
     */
    public function getContent()
    {
        /**
         * If values have been submitted in the form, process.
         */
        if (((bool)Tools::isSubmit('submitCaisseEnregistreuseModule')) == true) {
            $this->postProcess();
        }
        
        $this->context->smarty->assign('module_dir', $this->_path);
        
        $output = $this->context->smarty->fetch($this->local_path.'views/templates/admin/configure.tpl');
        
        return $output.$this->renderForm();
    }
    
    /**
     * Create the form that will be displayed in the configuration of your module.
     */
    protected function renderForm()
    {
        $helper = new HelperForm();
        
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);
        
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitCaisseEnregistreuseModule';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
        .'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        
        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFormValues(), /* Add values for your inputs */
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        );
        
        return $helper->generateForm(array($this->getConfigForm()));
    }
    
    /**
     * Create the structure of your form.
     */
    protected function getConfigForm()
    {
        return array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Settings'),
                    'icon' => 'icon-cogs',
                ),
                'input' => array(
                    array(
                        'col' => 3,
                        'type' => 'text',
                        //'prefix' => '<i class="icon icon-envelope"></i>',
                        'desc' => $this->l('Enter your cash register API key'),
                        'name' => 'CAISSEENREGISTREUSE_ACCOUNT_API',
                        'label' => $this->l('API key'),
                    ),
                    array(
                        'col' => 3,
                        'type' => 'text',
                        //'prefix' => '<i class="icon icon-envelope"></i>',
                        'desc' => $this->l('Enter your cash register shop ID'),
                        'name' => 'CAISSEENREGISTREUSE_ACCOUNT_SHOPID',
                        'label' => 'Shop ID',
                    ),
                    /*array(
                     'type' => 'password',
                     'name' => 'CAISSEENREGISTREUSE_ACCOUNT_PASSWORD',
                     'label' => $this->l('Password'),
                     ),*/
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                ),
            ),
        );
    }
    
    /**
     * Set values for the inputs.
     */
    protected function getConfigFormValues()
    {
        return array(
            'CAISSEENREGISTREUSE_ACCOUNT_API' => Configuration::get('CAISSEENREGISTREUSE_ACCOUNT_API', 'xxxxxxxx'),
            'CAISSEENREGISTREUSE_ACCOUNT_SHOPID' => Configuration::get('CAISSEENREGISTREUSE_ACCOUNT_SHOPID', null),
        );
    }
    
    /**
     * Save form data.
     */
    protected function logData($txt)
    {
        $f = fopen($this->local_path.'logBackground.txt',"a+");
        fwrite($f,date("m/d/Y H:i ").$txt."\r\n");
        fclose($f);
        //file_put_contents($this->local_path.'hookActionUpdateQuantity.txt', $data , LOCK_EX);
    }
    
    /**
     * Save form data.
     */
    protected function postProcess()
    {
        $form_values = $this->getConfigFormValues();
        foreach (array_keys($form_values) as $key) {
            Configuration::updateValue($key, Tools::getValue($key));
        }
    }
    
    /**
     * Add the CSS & JavaScript files you want to be loaded in the BO.
     */
    public function hookBackOfficeHeader()
    {
        if (Tools::getValue('module_name') == $this->name && false) {
            $this->context->controller->addJS($this->_path.'views/js/back.js');
            $this->context->controller->addCSS($this->_path.'views/css/back.css');
        }
    }
    
    /**
     * Add the CSS & JavaScript files you want to be added on the FO.
     */
    public function hookHeader()
    {
        $this->context->controller->addJS($this->_path.'/views/js/front.js');
        $this->context->controller->addCSS($this->_path.'/views/css/front.css');
    }
    //actionUpdateQuantity actionCustomerAccountUpdate actionCustomerAccountAdd actionProductSave actionPaymentConfirmation
    public function hookActionCustomerAccountUpdate($params)
    {
        $data = print_r($params,true);
        //$this->logData('hookActionCustomerAccountUpdate '. $data);
        
    }
    public function hookActionUpdateQuantity($params)
    {
        $data = print_r($params,true);
        //$this->logData('hookActionUpdateQuantity '. $data);
    }
    public function hookActionCustomerAccountAdd($params)
    {
        $data = print_r($params,true);
        //$this->logData('hookActionCustomerAccountAdd '. $data);
    }
    public function hookActionPaymentConfirmation($params)
    {
        $data = print_r($params,true);
        //$this->logData('actionPaymentConfirmation '. $data);
    }
    public function hookActionValidateOrder($params)
    {
        $accountKEYAPI = Configuration::get('CAISSEENREGISTREUSE_ACCOUNT_API', null);
        $shopID = Configuration::get('CAISSEENREGISTREUSE_ACCOUNT_SHOPID', null);
        $liveMode = true;//Tools::getValue('CAISSEENREGISTREUSE_LIVE_MODE',Configuration::get('CAISSEENREGISTREUSE_LIVE_MODE', null));
        $debugMode = true;//Tools::getValue('CAISSEENREGISTREUSE_DEBUG_MODE',Configuration::get('CAISSEENREGISTREUSE_DEBUG_MODE', null));
        
        if (!$liveMode) {
            if ($debugMode) file_put_contents($this->local_path.'LiveModeNotEnabled.txt',serialize($accountKEYAPI), LOCK_EX);
            return;
        }
        if ($accountKEYAPI===null || $accountKEYAPI=='xxxxxxxx' || $accountKEYAPI=='') {
            if ($debugMode) $this->logData('API KEY invalid '. $accountKEYAPI);
            return;
        }
        //file_put_contents($this->local_path.'hookActionValidateOrderWSConfig.txt',serialize($accountKEYAPI), LOCK_EX);
        
        
        $id_order = $params['order']->id;
        $status = $params['orderStatus']->id;
        
        $data = print_r($params,true);
        
        if ($debugMode) file_put_contents($this->local_path.'hookActionValidateOrder.txt', $data, LOCK_EX);
        
        $codeBarreClient = "Pr".$params['customer']->id;
        $modePaiement = $params['order']->payment;
        $productList = $params['order']->product_list;
        $customer = $params['customer'];
        
        $addedItems = "";
        foreach ($productList as $product) {
            $addedItems .= "&itemsList[]=|Pr".$product['id_product']."_".$product['cart_quantity']."_".rawurlencode($product['name'])."_".$product['total_wt']/$product['cart_quantity'];
        }
        
        $totalDiscount = $params['order']->total_discounts;
        if ($totalDiscount) {
            $addedItems .= "&itemsList[]=-1_1_".rawurlencode("Discount")."_-".$totalDiscount;
        }
        
        
        $shippingPrice = $params['order']->total_shipping;
        if ($shippingPrice) {
            $addedItems .= "&itemsList[]=-0_1_".rawurlencode("Shipping")."_".$shippingPrice;
        }
        
        //http:// &payment=-2&idUser=74&client[nom]=Nom &client[prenom]=Prï¿½nom &client[email]=Email &client
        // [telephone]=Tï¿½lï¿½phone &client[adresseligne1]=Adresse 1&client[adresseligne2]=Adresse 2&client[commentaireadresse]=Commentaire adresse &client[codepostal]=Code postal&client[ville]=Ville &
        // client[pays]=FR&client[numtva]=TVA &client[rcs]=RC &client[codeBarre]=951951&client[telephone2]=Tï¿½lï¿½phone2&deliveryMethod=4
        
        $infosClient = "";
        $infosClient .= "&client[nom]=".rawurlencode($customer->lastname);
        $infosClient .= "&client[prenom]=".rawurlencode($customer->firstname);
        $infosClient .= "&client[email]=".rawurlencode($customer->email);
        //$infosClient .= "&client[telephone]=".$customer->lastname;
        //$infosClient .= "&client[adresseligne1]=".$customer->lastname;
        //$infosClient .= "&client[adresseligne2]=".$customer->lastname;
        //$infosClient .= "&client[commentaireadresse]=".$customer->lastname;
        //$infosClient .= "&client[codepostal]=".$customer->lastname;
        //$infosClient .= "&client[ville]=".$customer->lastname;
        //$infosClient .= "&client[pays]=".$customer->lastname;
        //$infosClient .= "&client[numtva]=".$customer->lastname;
        $infosClient .= "&client[rcs]=".rawurlencode($customer->siret);
        $infosClient .= "&client[codeBarre]=Pr".$customer->id;
        //$infosClient .= "&client[telephone2]=".$customer->lastname;
        //$urlWS = "http://caisses/workers/webapp.php?idboutique=".$shopID."&key=".$accountKEYAPI."&paymentName=".rawurlencode($modePaiement)."&idUser=Prestashop".$infosClient."&deliveryMethod=1".$addedItems;
        $urlWS = "https://caisse.enregistreuse.fr/workers/webapp.php?idboutique=".$shopID."&key=".$accountKEYAPI."&paymentName=".rawurlencode($modePaiement)."&idUser=Prestashop".$infosClient."&deliveryMethod=1".$addedItems;
        
        if ($debugMode) $this->logData('Fetch url : '. $urlWS);
        
        $result = file_get_contents($urlWS);
        if ($result===false) {
            $error = error_get_last();
            if ($debugMode) $this->logData('HTTP request failed. Error was : '. $error['message']);
        }
        
        $recordResult = json_decode(str_replace("'", '"', $result));
        
        if ($recordResult) {
            if (property_exists($recordResult,'errorMessage')) {
                if ($debugMode) $this->logData('Server returned explicit error message : '. $recordResult->errorMessage);
            }
            if (property_exists($recordResult,'orderID')) {
                if ($debugMode) $this->logData('Order was successfully created : '. $recordResult->orderID);
            }
        }
    }
    public function hookDisplayPayment($params)
    {
        /* Place your code here. */
    }
    
    public function hookDisplayPaymentReturn()
    {
        /* Place your code here. */
    }
}
